/*   FILE NAME : EXAMP4.C                                        */
/*   Program Example : AD500 PMU PROGRAMMING                     */
/*                                                               */
/*   Purpose : This program measures 16 channel of voltages      */
/*             and display them. If the voltage of any channel   */
/*             is greater than a certain level then it close a   */
/*             relay to drive an alarm.                          */
/*                                                               */
/*   Remark :  The AD500 has a 16 channel multiplexer in slot    */
/*             0 and 16 channel relay actuator in slot 1. The    */
/*             voltage measurement is done by an HP3478A DMM.    */
/*             The AD500 has an address of 9 and HP3478A has an  */
/*             address of 23.                                    */
/*                                                               */
/*   Language : C86                                              */
/*                                                               */
#include <stdio.h>
#include <math.h>
extern double atof();

int ioerr;

main(){
  char oeolstr[8],buff1[100],buff2[100],buff3[100];
  char *instr,*outstr,*chstr;
  double v[16];
  float v_lmt;
  int i,k,hp3478,ad500,alarm;

  hp3478=23;
  ad500=9;
  v_lmt=2.0;   /* Voltage limit */
  oeolstr[0]=13;
  instr=buff1;
  chstr=buff3;

  crt_cls();
  ieseg(0xC000);
  ieabort();

  for(i=1;i<10000;i++)i=i+1-1;   /* Dealy for AD500 reset  */
  ietimout(5000);                /* Setting timeout 5 sec. */
  ieeol(ad500,2,oeolstr,1,0,10); /* Init AD500A terminator */

  /* Set the DVM and Multiplexer */
  outstr="F1T3RAN5";
  ioerr=ieoutput(hp3478,outstr,8); /* Send DVM setting string */
  iochk();                       /* Error check             */
  outstr="DW0,16;DW1,0";
  ioerr=ieoutput(ad500,outstr,12); /* Open relays of Mux & Actuator */
  iochk();                       /* Error check             */

  /* Measurement start */
  while(1==1){
    alarm=0;
    for(i=0;i<16;i++){
      itoa(i,chstr);
      outstr=buff1;
      strcpy(outstr,"DW0,");
      strcat(outstr,chstr);
      ioerr=ieoutput(ad500,outstr,strlen(outstr)); /* Close ch I  */
      iochk();
      for(k=0;k<100;k++)k=k;      /* Delay for the relay operation. */

      ietrig(hp3478);             /* Trigger the DVM.        */
      ioerr=ieenter(hp3478,instr,100);  /* Enter DVM reading */
      iochk();                    /* Error check             */

      if(instr[0]=='+')instr[0]=32;  /* C86 atof() doesn't like '+' */
      v[i]=atof(instr);
      printf("\nCHANNEL %2d   %12f VOLTS",i,v[i]);
      if(v[i] > v_lmt){
         printf("    ALARM !!");
         alarm=1;
      }
    }
    if(alarm==1){
      outstr="DW1,1";
      ieoutput(ad500,outstr,5);     /* Set the alarm  */
      printf("\n\nSet Alarm ! \n\n");
    }
    else {
      outstr="DW1,0";
      ieoutput(ad500,outstr,5);     /* Reset the alarm  */
      printf("\n\nReset Alarm ! \n\n");
    }
  }
}

iochk()  /*  I/O error check routine  */
{
  switch(ioerr){
    case 0 : break;
    case 1 : printf("\nTIME OUT !!");exit(0);
    default : printf("\nINTERFACE ERROR !!");exit(0);
  }
}
