/*  FILE NAME : EXAMP6.C                                         */
/*  Program Example : INTERRUPT HANDLING                         */
/*                                                               */
/*  Purpose : This program measures 100 voltage readings and     */
/*            displays them. It also goes to service subroutines */
/*            when interrupts happen. You can press the 'SRQ'    */
/*            key to generate an SRQ.                            */
/*  Remark  : This program is written for the HP3478A DMM. If    */
/*            other model of voltmeters are used, please check   */
/*            the operating manual and make necessary modifi-    */
/*            cation to this program.                            */
/*            In this example, the program only shows when the   */
/*            interrupt happens. You can add more actions to     */
/*            the service routine to response to an interrupt.   */
/*  Caution : This program is for C86 only                       */
/*                                                               */
#include <stdio.h>
#define STACK 5000
#define GPIBPORT 0x2B8
#define STSPORT 0x2BA
#define DVM 23
#define GPIBSRQ 0xA          /* IRQ2 */
int srqflag,ioerr;

srqsrv(){
  srqflag=1;
}

main(){
  int i;
  char *cmdstr,instr[100];
  crt_cls();
  printf("\n ************  SRQ TEST  ************ \n");

  ieseg(0xC000);
  ieinit(GPIBPORT,21,16);    /* Enable IRQ2  */
  ieabort();

  /* Set the DVM & UNMASK Front Panel SRQ bit */
  cmdstr="KM20F1T3RAN5";
  ioerr=ieoutput(DVM,cmdstr,strlen(cmdstr)); /* Set instrument */
  iochk();

  /* Enable interrupt of SRQ  */
  intrinit(srqsrv, STACK, GPIBSRQ);
  inportb(STSPORT);           /* Clear 7210 intr status  */

  /* Measurement start        */
  for(i=1;i<101;i++){
    ietrig(DVM);              /* Trigger the DVM.     */
    ioerr=ieenter(DVM,instr,100); /* Enter DVM reading    */
    iochk();
    printf("%3d  %s",i,instr);
    if(srqflag==1)srqhand();
  }
  intrrest(GPIBSRQ);
}


iochk()  /*  I/O error check routine  */
{
  switch(ioerr){
    case 0 : break;
    case 1 : printf("\nTIME OUT !!");exit(0);
    default : printf("\nINTERFACE ERROR !!");exit(0);
  }
}


srqhand(){
  srqflag=0;
  printf("\nSRQ HAPPENED !!! \n");
  outportb(0x20,0x20);  /* Enable 8259  */
  iespoll(DVM);
  inportb(STSPORT);
}

/* End of program  */


