/*-----------------------------------------------------------------------*/
/*                              PC1ADEV.C                                */
/*                                                                       */
/* Date:    June 21, 1989                                                */
/* Author:  Kathy K. F. PU                                               */
/* ver:     1                                                            */
/*                                                                       */
/* This demo program makes PC2 card function as a device that sends out  */
/* data and receives data to and from the bus controller.                */
/*                                                                       */
/* Data transfer is done by polling the 7210 address status register to  */
/* see whether my listen address or my talk address has been sent to the */
/* bus by the bus controller. If so, then the PC2 does a data transfer.  */
/*                                                                       */
/* When you program for the bus controller, remember that a UNT has to   */
/* sent after making pc1a a talker and receiving its talked data.        */
/*                                                                       */
/* Inside the program, user can hit RETURN key to request for a keyboard */
/* input to the ring buffer. If RETURN key hit and PC1A is not addressed */
/* as a talker, SRQ  will be generated to interrupt controller. So a     */
/* service routine needs to be programmed for the controller.            */
/*                                                                       */
/* For your program to work properly, you have to pay attentin to:       */
/*                                                                       */
/*      1.     : This program uses ANSI device driver for the screen.    */
/*               A command in the form of device=pathname\ansi.sys needs */
/*               to be put into the config.sys file which is used during */
/*               boot up time.                                           */
/*                                                                       */
/*      2.     : If you use the demo program source code to develope your*/
/*               programs, note that these four string literals, TALK_S, */
/*               LISTEN_S, SEC0DATA_S, SEC0_S, always end up with exactly*/
/*               one space.                                              */
/*                                                                       */
/*      3.     : PC1A memory segment switch  has to be set to hex d000   */
/*               which means A18 and A16 are set to '1', and other to '0'*/
/*                                                                       */
/*      4.     : PC1A can be switched from listener to talker directly.  */
/*                                                                       */
/*      5.     : PC1A can not be switched from a talker to listerner,    */
/*               the controller has to send a UNT to untalk it.          */
/*                                                                       */
/*-----------------------------------------------------------------------*/
#include    <stdio.h>
#include    <dos.h>

#define     ESCkey           27
#define     RTNkey           13

#define     ioport            0x2b8
#define     setting           0x20

/*----------------ansi control sequence for the screen-------------------*/
#define     cursor(x,y)      printf("%c[%d;%dH", '\033',x,y)
#define     cleol            printf("%c[K", '\033')

int         ioerr;

clear_screen()

{
printf("\x1b[2J");               /* ansi control sequence for screen */
}


get_key()                        /* functin to get the keyhit */
{
union REGS r;
r.h.ah=0;
return int86(0x16, &r, &r);
}

bioskey(c)

int c;

{
switch(c)
  {
   case 0:  return get_key();
   case 1:  return kbhit();
  }
}
/*------------------------------ckkey----------------------------------------*/
/*   if no kbhit, do nothing. if kbhit, then get the key, test it            */
/*   and return the ascii code.                                              */
/*---------------------------------------------------------------------------*/

 int ckkey()

{
  union inkey
   {
    char   ch[2];
    int    i;
   }  c;

 if (kbhit())
   {
      c.i=bioskey(0);                     /* read the key      */
      if (c.ch[0])                        /* if is normal ascii key*/
         {
           switch (c.ch[0])
            {
              case ESCkey:  return  27;
              case RTNkey:  return  13;
              default    :  return   1;
            }
         }
      else                  return   1;

    }
}

iochk()                          /* I/O error check function         */
{
  switch(ioerr)
   {
    case  0 : break;
    case  1 : printf("\nTIME OUT !!");
              break;
    default : printf("\nINTERFACE ERROR !!");
   }
}


t_delay(n)                       /*  function for delay   */

int n;

{
int i,j;
j=10 * n;
for (i=1; i<=j; i++) i=i+1-1;
}

main()

{
static char outeols[]= {13,10,'\0'};/* output string terminator is 13, 10  */
static int  buffer_head=0;
static int  buffer_tail=0;
static int  ta=0;
static int  la=0;
static int  read_count=0;
static int  write_count=0;
static int  lastbufferno=4;
typedef char string50[50];
string50    ringbuffer[5];
int         myaddr;
static int  ineolbyte=10;    /* input string terminator is 10  */
static int  outeol=0;        /* terminated w/oeolstr only */
static int  ineol=0;         /* terminated when ineolb rcvd or EOI true */
static int  oeols_length=2;
char        input_s[150];
int         length;
int         addr_status;
int         keyhit;
int         iocount;
int         i;

clear_screen();

cursor(1,1);
printf("rev. 1    6/21/89");

cursor(2, 25);
printf(">>>Demo PC1A as a device <<<");

cursor(4,1);
printf("After you enter the PC1A's device address, ");

cursor(5,1);
printf("If need keyboard input, press RETURN. If want to exit, press ESC");

cursor(6,1);
printf("Enter device address for pc1a. Use address other than 0 or 21 : ");
scanf("%d",&myaddr);              /* input the device address for 4834  */

ieseg(0xd000);
ieinit(ioport, myaddr, setting);   /* initialize  as a device, normalspeed  */

ioerr=ietimout(0);                 /* set time out period */
iochk();
ieeol(myaddr,outeol,outeols,oeols_length,ineol,ineolbyte);


keyhit=0;

while (keyhit!=ESCkey)

{
 keyhit=0;
 {
 addr_status=inp(ioport+ 4);

if (addr_status==0)
  outp(ioport +4, 0x31);            /* add mode  */

 la= (addr_status & 0x10)||(addr_status & 0x04);      /* get LPAS */
 ta= ((addr_status & 0x08)||(addr_status & 0x02));    /*get TPAS */
 }

 if (la)                                  /* if addressed as listener */
   {
    iocount=0;

    for (i=8; i<=17; i++)
     {  cursor(i, 1); cleol;
     }
    while (!iocount)                      /* wait until data be entered from controller */
    {
    ieenter(-1, input_s, 150);
    iocount=iestatus(9);
    }
    cursor(8, 1);
    printf("display string read from controller as : ");
    cursor(9,1);
    printf(input_s);
   }

if (ta)                                   /* if addressed as talker */
 {
    read_count++;
    for (i=8; i<=17; i++)
     {  cursor(i, 1); cleol;
     }
    if (read_count <= write_count)       /* if buffer still have data  */
     {                                   /* advance head and output to bus */
      buffer_head++;
      if (buffer_head>lastbufferno)  buffer_head=0; /* wrap around  */
      length= strlen(ringbuffer[buffer_head]);
      ieoutput(-1, ringbuffer[buffer_head], length);
     }
   else                                  /* else if read_count > write_count */
     {                                   /* which means buffer become empty  */
                                         /* input from keyboard to write buffer */
                                         /* then advance write_count, head_count, tail_count*/
      buffer_head++;  if (buffer_head>lastbufferno) buffer_head=0;
      buffer_tail++;  if (buffer_tail>lastbufferno) buffer_tail=0;
      write_count++;

      cursor(11, 1);
      printf("You are addressed as a talker, but you have no data in buffer.");
      cursor(12,1);
      printf("Enter data: ");
      scanf("\n%[^\n]", ringbuffer[buffer_tail] );
      length= strlen(ringbuffer[buffer_head]);
      ieoutput(-1, ringbuffer[buffer_head], length);

     }
  cursor(14,1); printf("Send data in ringbuffer to bus as: ") ;
  cursor(15,1); printf("%s", ringbuffer[buffer_head]);
  cursor(16,1);
  printf("read count that counts strings read from buffer and output to bus = %d ", read_count);
  cursor(17,1);
  printf("write count that counts strings entered from keyboard to buffer= %d ", write_count);
 }

keyhit=ckkey();                    /* check if key hit */

if (keyhit==RTNkey)
 {
  if (!((buffer_tail==buffer_head) && (write_count >read_count) )) /* if not buffer full */
   {
    write_count++;                 /* do input */
    buffer_tail++;  if (buffer_tail>lastbufferno) buffer_tail=0;
    cursor(20,1);
    printf("Enter keyboard input to ring buffer:");
    cursor(21,1);
    scanf("\n%[^\n]", ringbuffer[buffer_tail]);
    cursor(20,1); cleol;
    cursor(21,1); cleol;
  }

 if (!ta)                   /* if keyhit and not addressed as a talker, SRQ int */
 outp(ioport +3, 0x41 );    /* generate SRQ int by rsv=1  and bit 0 = 1 */

}
}  /* forever loop while not ESCkey */
} /* main */
