/*-----------------------------------------------------------------------*/
/*                              PC1AINT.C                                */
/* Date:    6/21, 1989                                                   */
/* Author:  Kathy K. F. PU                                               */
/* ver:     1                                                            */
/*                                                                       */
/* This program demonstrates how a 4834 use SRQ to interrupt PC/AT. It   */
/* uses interrupt vector 0x0f for iRQ7.  The vector is redirected to     */
/* the new interrupt service routine.                                    */
/*                                                                       */
/* If you run this program under Microsoft C, keep the statement         */
/* #define  MSC 1. If you run under Turbo C,  then you have either       */
/* delete this statement, or mark this statement as comment.             */
/*                                                                       */
/*      1.     : This program uses ANSI device driver for the screen.    */
/*               A command in the form of device=pathname\ansi.sys needs */
/*               to be put into the config.sys file which is used during */
/*               boot up time.                                           */
/*                                                                       */
/*      2.     : If you need to use the source code, mind that TALK_S,   */
/*               LISTEN_S,SEC0DATA_S, SEC0_S,  these four strings always */
/*               end with exactly one space.                             */
/*                                                                       */
/*      3.     : PC1A memory segment switch  has to be set to hex d000   */
/*               which means A18 and A16 are set to '1', and other to '0'*/
/*                                                                       */
/*      4.     : The program does not do a complete testing of all       */
/*               possible conditions for the 4834 device status.         */
/*               But, you need to consider all exceptional cases in your */
/*               program or in your interrupt service routine. Or else,  */
/*               The whole system may hang up.                           */
/*                                                                       */
/*-----------------------------------------------------------------------*/
#include    <stdio.h>
#include    <dos.h>

#define     MSC
#define     TALK_S           "UNL UNT MLA TALK "
#define     LISTEN_S         "UNL UNT MTA LISTEN "
#define     UNT_S            "UNT"
#define     UNL_S            "UNL"
#define     SEC0DATA_S       "SEC 0 DATA "
#define     SEC0_S           "SEC 0 "
#define     ioport            0x2b8
#define     irq_7             0x1c
#define     irq_7_vec         0xf

/*----------------ansi control sequence for the screen-------------------*/
#define     cursor(x,y)      printf("%c[%d;%dH", '\033',x,y)
#define     cleol            printf("%c[K", '\033')

int         talks_length;
char        talkstr[20];
char        cmdstr[150];
char        datas[150];
char        output_s[150];
char        devs[3];
int         ioerr;
int         length;
int         addr;
int         intcount; /* count for interrupt service routine entered */
int         isrdone;  /* flag for marking the interrupt service routine done */
int         smr;      /* serial message received flag obtained from iespoll */
int         s;        /* serial poll response */

void        interrupt  far newint();

clear_screen()

{
printf("\x1b[2J");               /* ansi control sequence for screen */
}

iochk()                          /* I/O error check function         */
{
  switch(ioerr)
   {
    case  0 : break;
    case  1 : printf("\nTIME OUT !!");
              break;
    default : printf("\nINTERFACE ERROR !!");
   }
}


t_delay(n)                       /*  function for delay   */

int n;

{
int i,j;
j=10 * n;
for (i=1; i<=j; i++) i=i+1-1;
}


set_up_4834(config_s, dev)       /*output configuration command to device*/

char   *config_s;
int    dev;

{

strnset(cmdstr,' ',150);         /* blank the array used for holding commands */
length=strlen(LISTEN_S);
strncpy(cmdstr, LISTEN_S, length); /* set up the listen string  */
itoa(dev, devs, 10);               /*convert decimal integer address to ascii */
strncat(cmdstr, devs, 1);        /*  append the ascii of device address to it */
length=strlen(cmdstr);
cmdstr[length]=' ';              /* append a space to the command string */
cmdstr[length+1]='\0';           /* append a null char */

strncat(cmdstr,SEC0DATA_S,11);   /* append a "SEC 0 DATA " */
length=strlen(cmdstr);
iesend(cmdstr,length);           /* output the command to set up device 2ndary */
length=strlen(config_s);         /* address as listen address   */
ioerr=ieoutput(-1, config_s,length); /*output the configuration command as data*/
iochk();
iesend(UNL_S,3);                 /* unaddress all listeners */

}


read_4834_setup(dev)             /* function to read the 4834 configuration  */
                                 /* and display it on the screen  */
int dev;

{
strnset(cmdstr,' ',150);        /* blank the array for holding commands */
strnset(datas,' ',150);         /* blank the array for holding data     */
itoa(dev, devs, 10);            /* convert the decimal address to ascii */

/* set up device secondary address as talk address */

length=strlen(TALK_S);
strncpy(cmdstr,TALK_S,length);  /* set up device address as talker address */
strncat(cmdstr,devs,1);         /* append the ascii code of device address */
length=strlen(cmdstr);
cmdstr[length]=' ';             /* append a space */
cmdstr[length+1]='\0';          /* append a null char */

strncat(cmdstr,SEC0_S,6);       /* append a string of "SEC 0 " */
length=strlen(cmdstr);
iesend(cmdstr,length);          /* out put the command to set up talk address */
ieenter(-1,datas, 150);         /* enter configuration data from device */
printf("%s\n", datas);          /* display the entered configuration data */
iesend(UNT_S,3);                /* unaddress the talk address */
}

set_talkstr()                   /* set up command string which set up the  */
{                               /* talk address that is used by ieenter in */
char addrs[3];                  /* the interrupt service routine   */
int  length;

itoa(addr, addrs, 10);          /* convert device address to string   */
length=strlen(TALK_S);
strncpy(talkstr, TALK_S,length);

length=strlen(addrs);
strncat(talkstr, addrs, length);
talks_length=strlen(talkstr);

}

void interrupt  far newint()

/*   start of interrupt serivce routine for 4834 SRQ interrupt    */

{

intcount++;
isrdone=1;                      /* set up isrdone flag */

s=iespoll(addr);                /* do a serial poll   */
smr=s & 0x01;                   /* test serial message received bit   */
if (smr)
  {
  iesend(talkstr, talks_length);/* send command to IEEE bus to set up talk add*/
  ieenter(-1, datas,150);       /* for the next ieenter function */
  }
#ifdef MSC                      /* if run under microsoft C compiler */
 {
  outp(0x20, 0x20);             /* EOI for 8259 to enable int <= current int  */
  inp(ioport+2);                /* read int status register to clear irq  */
 }
#else                           /* if run under Turbo C  */
 {
  outportb(0x20, 0x20);         /* EOI for 8259 to enable int <= current int  */
  inportb(ioport+2);            /* read int status register to clear irq  */
 }
#endif
}

/*--------------------------------------------------------------------------*/
/*                              Main  Program                               */
/*--------------------------------------------------------------------------*/

main()

{
char     far  *old_vector;
char     far  *new_vector;

struct   address{
                 char far *p;
                };
struct  address  far *add= (struct address far * ) (0xf * 4);
                                /* declare and initialize the add to point to */
                                /* memory address 0x3c, which is the address */
                                /* that contains the vector 0f for irq 7 */
char     config_s[150];
char     continu;
int      ineolbyte;
int      outeol;
int      ineol;
int      oeols_length;
static char outeols[]= {13,10,'\0'};
int      i;
char     use_default;
char     config_ok;

isrdone=0;
intcount=0;
ieseg(0xd000);
ieinit(ioport, 21, irq_7);      /* enable irq7 by  0x1c=0000 0000 0001 1100 */
#ifdef MSC
  outp(ioport+2, 0x40);         /* enable srq interrupt in 7210 */
#else
  outportb(ioport+2, 0x40);     /* enable srq interrupt in 7210 */
#endif

/*the following tow lines works for microsoft C only,turboc has its counter part*/
/* oldint=_dos_getvect(irq_7_vec);*/     /* save old int vector    */
/*_dos_setvect(irq_7_vec, newint);*/     /* install new int vector */

/* following vector change works for both microsoft C and Turbo C    */
old_vector = add->p;            /* save the vector pointed by 003c to old_vector*/
add->p = (char far *) newint;   /* make the 003c point to newint */
clear_screen();
ioerr=ietimout(35000);          /* set time out period */
iochk();

cursor(1,1);
printf("rev. 1    6/21/89");
cursor(2, 25);
printf(">>>DEMO PC1A INTERRUPT FUNCTION<<<");
cursor(4,1);
printf("Enter 4834 device address. Use address other than 0 or 21 : ");
scanf("%d",&addr);              /* input the device address for 4834  */
set_talkstr();                  /* set up command string used by iesend in newint */
ieabort();
oeols_length=strlen(outeols);
outeol=2;
ineol=0;
ineolbyte=13;
/*---------------------------------------------------------------------------*/
/*   ieeol is for setting terminators for input and output string, so that   */
/*   pc1a and 4834 can communicate with each other.                          */
/*                                                                           */
/*   outeol=2  terminated w/outeols only and not w/eoi                       */
/*   outeols   {13,10,'\0'} the string to mark the end of output string      */
/*   ineol =0  terminated when ineolb rcvd or EOI true or input string full  */
/*   ineolbyte the char upon which the input string will be terminated       */
/*                                                                           */
/*---------------------------------------------------------------------------*/
ieeol(addr,outeol,outeols,oeols_length,ineol,ineolbyte);
set_up_4834("T1, A0", addr);    /* set local loop mode */

cursor(6,1);
printf("Display default 4834 configuration as follows: ");
cursor(7,1);
read_4834_setup(addr);
cursor(8,1);
printf("Always keep 'T1, A0 ' for this loop back interrupt test.  ");
cursor(9,1);
printf("Use default,  y/n ? :  ");
scanf("\n%c",&use_default);
 if ((use_default=='n')||(use_default=='N'))  /* if not using default  */
{
  config_ok='n';
  while ((config_ok=='n')||(config_ok=='N'))  /* looping until config_ok */
   {
    for (i=6; i<=11; i++)                     /* clear lines */
     {
       cursor(i, 1); cleol;
     }
      cursor(6,1);
      printf("You need to have 'T1, A0' in your configuration. ");
      cursor(7,1);
      printf("Enter configuration : ");
      strnset(config_s,' ', 150);             /* blank the old config. string */
      scanf("\n%[^\n]", config_s);            /* input new configuration data */
      set_up_4834(config_s, addr);            /* set up for new configuration */

      cursor(8,1);
      printf("Current 4834 configuration is :");
      cursor(9,1);
      read_4834_setup(addr);                  /* read and display configuration*/

     cursor(11,1);
     printf("Is configuration ok? y/n ");
     scanf("\n%c",&config_ok);

   }  /*--while loop---*/                     /* end of looping for config. change*/
     cursor(6,1);    cleol;
     cursor(11,1);   cleol;

}  /*-----if use_default==n--------*/

continu='y';

iedevclr(addr);                               /* clear 4834 buffer before test */
t_delay(1000);
while((continu!='n') && (continu!='N'))

{

  strnset(datas,' ',150);                     /* blank the data string */
  strnset(output_s,' ',150);
  cursor(12,1);
  printf("Enter a test string to be sent to 4834: ");
  cursor(13,1);
  scanf("\n%[^\n]",output_s);                 /* input a data string   */
  length=strlen(output_s);
  ioerr=ieoutput(addr, output_s, length);     /* send it out to 4834   */
  iochk();

  while (! isrdone);                          /* wait for interrupt completion*/
  isrdone=0;                                  /* reset the interrupt flag */
  cursor(16,1);
  printf("Current interrupt count is: %d ", intcount);
  cursor(17,1);
  printf("Serial poll response before ieenter is  %x", s );
  cursor(18,1);
  printf("The  data read from 4834 by interrupt function is : ");

  cursor(19,1);                               /* if the isr tested serial */
                                              /* message received */
  if (smr)
    printf("%s", datas);                      /* display the data entered from 4834*/
  else
    printf("4834 data not received                           ");
  cursor(23,1);
  printf("Continue the loop back test? y/n: ");
  scanf("\n%c",&continu);
  if ((continu=='y')||(continu=='Y'))
     {
       for (i=11; i<=23; i++)
        {
         cursor(i,1); cleol;
        }
     }
}  /*  while continu!='n' and continu!='N' */
                                             /* restore old vector when done */
/* _dos_setvect(irq_7_vec, oldint); */       /* works for microsoft C only */
add->p=old_vector;                           /* works for both MSC and turbo c*/

}  /*  main */



