
/*-----------------------------------------------------------------------*/
/*  Date       : 6/21/89                                                 */
/*                                                                       */
/*  Programmer : Kathy K. F. Pu                                          */
/*  Version    : 1                                                       */
/*                                                                       */
/*  File Name  : PC1A_34.C                                               */
/*               DATA LOOP BACK TEST USING A 4834 SERIAL INTERFACE UNIT  */
/*               AND A PC1A CARD                                         */
/*                                                                       */
/*  Purpose    : This program sends out data string to a channel of 4834,*/
/*               and reads the looped back data from the same channel and*/
/*               displays it.                                            */
/*                                                                       */
/*               This program is written for the 4834 serial interface   */
/*               unit used to connct devices with RS-232 or RS-449 serial*/
/*               interface to the IEEE 488 bus.                          */
/*                                                                       */
/*  Languages  : Microsoft C 5.1 and Borland Turbo C 2.0                 */
/*                                                                       */
/*  Remark     : Please pay attention to the following, or else it will  */
/*               not work appropriately.                                 */
/*                                                                       */
/*      1.     : In the program, "T1" is already sent to the secondary   */
/*               address of 4834 to select the local loop mode, which    */
/*               is necessary for this test.  Don't change it to "T0".   */
/*                                                                       */
/*      2.     : This program uses ANSI device driver for the screen i/o.*/
/*               A command in the form of device=pathname\ansi.sys needs */
/*               to be put into the config.sys file which is used during */
/*               boot up time.                                           */
/*                                                                       */
/*      3.     : If you need to use the source code, mind that TALK_S,   */
/*               LISTEN_S,SEC0DATA_S, SEC0_S,  these four strings always */
/*               end with exactly one space.                             */
/*                                                                       */
/*      4.     : PC1A memory segment switch  has to be set to hex d000   */
/*               which means A18 and A16 are set to '1', and other to '0'*/
/*-----------------------------------------------------------------------*/
#include    <stdio.h>

#define     TALK_S           "UNL UNT MLA TALK "
#define     LISTEN_S         "UNL UNT MTA LISTEN "
#define     UNT_S            "UNT"
#define     UNL_S            "UNL"
#define     SEC0DATA_S       "SEC 0 DATA "
#define     SEC0_S           "SEC 0 "

/*----------------ansi control sequence for the screen-------------------*/
#define     cursor(x,y)      printf("%c[%d;%dH", '\033',x,y)
#define     cleol            printf("%c[K", '\033')

char        cmdstr[150];
char        datas[150];
char        devs[3];
char        config_s[150];
char        output_s[150];
int         ioerr;
int         length;


clear_screen()

{
printf("\x1b[2J");               /* ansi control sequence for screen */
}

iochk()                          /* I/O error check function         */
{
  switch(ioerr)
   {
    case  0 : break;
    case  1 : printf("\nTIME OUT !!");
              break;
    default : printf("\nINTERFACE ERROR !!");
   }
}


set_primary_talk_add(dev)        /* set talk address on the interface bus */

int dev;

{
strnset(cmdstr,' ',150);         /* blank the array used for holding commands */
strnset(datas,' ',150);          /* blank the array used for holding data    */
itoa(dev, devs, 10);             /* convert the device address to its ascii */

/* set up string of talk address  */

length=strlen(TALK_S);
strncpy(cmdstr,TALK_S,length);
strncat(cmdstr,devs,1);          /* append the ascii code of device address to */
length=strlen(cmdstr);           /* the command string  */
iesend(cmdstr,length);           /* output the command string to the bus */

}


set_up_4834(config_s, dev)       /*send configuration command to 4834*/

char   *config_s;
int    dev;

{

strnset(cmdstr,' ',150);         /* blank the array used for holding commands */
length=strlen(LISTEN_S);
strncpy(cmdstr, LISTEN_S, length); /* set up the listen string  */
itoa(dev, devs, 10);             /* convert decimal integer address to ascii */
strncat(cmdstr, devs, 1);        /* append the ascii of device address to string */
length=strlen(cmdstr);
cmdstr[length]=' ';              /* append a space to the command string */
cmdstr[length+1]='\0';           /* append a null char */

strncat(cmdstr,SEC0DATA_S,11);   /* append a "SEC 0 DATA " to command string*/
length=strlen(cmdstr);
iesend(cmdstr,length);           /* output the command to set up device 2ndary */
length=strlen(config_s);         /* address as listen address   */
ioerr=ieoutput(-1, config_s,length); /*output the configuration command as data*/
iochk();
iesend(UNL_S,3);                 /* unaddress all listeners */

}


read_4834_setup(dev)            /* read 4834 configuration and display it */

int dev;
{
strnset(cmdstr,' ',150);        /* blank the array for holding commands */
strnset(datas,' ',150);         /* blank the array for holding data     */
itoa(dev, devs, 10);            /* convert the decimal address to ascii */

/* set up device secondary address as talk address */

length=strlen(TALK_S);          /* get the length of TALK_S */
strncpy(cmdstr,TALK_S,length);  /* set up command string to assign bus talker */
strncat(cmdstr,devs,1);         /* append the ascii code of device address */
length=strlen(cmdstr);
cmdstr[length]=' ';             /* append a space */
cmdstr[length+1]='\0';          /* append a null char */

strncat(cmdstr,SEC0_S,6);       /* append a string of "SEC 0 " for secondary add*/
length=strlen(cmdstr);
iesend(cmdstr,length);          /* out put the command to set up talk address */
ieenter(-1,datas, 150);         /* enter configuration data from device */
printf("%s\n", datas);          /* display the entered configuration data */
iesend(UNT_S,3);                /* unaddress the talk address */
}
/*-----------------------------------------------------------------------*/
/*                            Main  Program                              */
/*-----------------------------------------------------------------------*/
main()

{

int             ineolbyte;
int             outeol;
int             ineol;
int             oeols_length;
static char outeols[]= {13,10,'\0'};
char            do_spoll;
char            do_ppoll;
int             addr;
char            continu;
int             s;             /* serial poll result */
int             p;             /* parallel poll result */
int             i;
char            use_default;
char            config_ok;

ieseg(0xd000);
ieinit(0x2b8, 21, 0x0);
clear_screen();
ioerr=ietimout(35000);         /* set time out period */
iochk();

cursor(1,1);
printf("rev. 1    6/21/89");

cursor(2, 25);                 /* print title   */
printf(">>>DEMO PC1A DATA LOOP BACK TEST <<<");

cursor(4,1);
printf("Enter 4834 device address. Use address other than 0 or 21 : ");
scanf("%d",&addr);             /* input the device address for 4834  */


ieabort();
ioerr=iedevclr(addr);          /* clear interface bus and 4834  buffer  */
ieppollc(addr, 0x8);           /* response a '1' on line 0 for 4834 ready */

oeols_length=strlen(outeols);
outeol=2;
ineol=0;
ineolbyte=13;

/*----------------------------------------------------------------------------*/
/*   use ieeol to set up terminators for pc1a's input string and output string*/
/*   properly, so pc1a can communicate with 4834                              */
/*                                                                            */
/*   outeol=2  terminated w/outeols only and not w/eoi                        */
/*   outeols   {13,10,'\0'} the string to mark the end of output string       */
/*   ineol =0  terminated when ineolb rcvd or EOI true or input string full   */
/*   ineolbyte the char upon which the input string will be terminated        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
ieeol(addr,outeol,outeols,oeols_length,ineol,ineolbyte);

set_up_4834("T1", addr);       /* set local loop mode */

cursor(6,1);
printf("Display default 4834 configuration as follows: ");
cursor(7,1);
read_4834_setup(addr);
cursor(8,1);
printf("Use default? you need 'T1' for loop test. enter y/n:  ");
scanf("\n%c",&use_default);
 if ((use_default=='n')||(use_default=='N'))  /* if not using default  */
{
  config_ok='n';
  while ((config_ok=='n')||(config_ok=='N'))  /* looping until config_ok */

   {
      cursor(10,1);
      printf("Enter configuration : ");
      strnset(config_s,' ', 150);             /* blank the old config. string */
      scanf("\n%[^\n]", config_s);            /* input new configuration data */
      set_up_4834(config_s, addr);            /* set up for new configuration */

      cursor(12,1);
      printf("Display 4834 configuration as:");
      cursor(13,1);
      read_4834_setup(addr);                  /* read and display configuration*/

     cursor(15,1);
     printf("Is configuration ok? y/n ");
     scanf("\n%c",&config_ok);

    for (i=9; i<=15; i++)                     /* clear lines */
     {
       cursor(i, 1); cleol;
     }
   }  /*--while loop---*/                     /* end of looping for config.data*/
}  /*-----if use_default==n--------*/


continu='y';

iedevclr(addr);                               /* clear 4834 buffer before test */

while((continu!='n') && (continu!='N'))       /* loop for data loop back testing*/

{
  cursor(12,1);
  printf("Enter a test string to be sent to 4834: ");
  strnset(output_s,' ',150);
  cursor(13,1);
  scanf("\n%[^\n]",output_s);                 /* input a data string to be  */
  length=strlen(output_s);                    /* sent to the 4834      */
  ioerr=ieoutput(addr, output_s, length);     /* send it out           */
  iochk();

  cursor(14,1);
  printf("Serial poll result is DIO line 8 be a 0 when in local loop mode: ");

  cursor(15,1);
  printf("Serial Poll? y/n : ");
  scanf("\n%c",&do_spoll );

  if ((do_spoll!='n') && (do_spoll!='N'))
    {
     s=iespoll(addr);                         /* do a serial poll */
     cursor(16,1);
     printf("4834 serial poll response byte in hex = %x ", s);
    }

  set_primary_talk_add(addr);                 /* set up 4834 address as talker*/
  strnset(datas,' ',150);
  ieenter(-1, datas,150);                     /* enter 4834 looped back data */
  cursor(17,1);
  printf("Display data input from 4834: ");
  cursor(18,1);
  printf("%s", datas);                        /* display the data */


  /*-parallel poll response should always be a 0 if in  local loop mode*/
  /*-means the device is not ready. It is programmed to response a '1'if ready*/
  cursor(20,1);
  printf("Result of parallel poll should be DIO line 0 be 0 if in local loop");
  cursor(21,1);
  printf("Parallel poll ? y/n: ");
  scanf("\n%c", &do_ppoll);

  if ((do_ppoll!='n') && (do_ppoll!='N'))
   {
    p=ieppoll();                              /* do a parallel poll */
    cursor(23,1);
    printf("Parallel poll response byte in hex = %x ", p);

   }
  cursor(24,1);
  printf("Continue the loop back test? y/n: "); /* ask if to continue the test*/
  scanf("\n%c",&continu);
  if ((continu=='y')||(continu=='Y'))
  {
     for (i=11; i<=24; i++)                   /* clean lines */
      {
       cursor(i,1); cleol;
      }
  }
}  /*  while continu!='n' and continu!='N' */

}  /*  main */



