{ FILE NAME : EX4_MS.PAS					}
{	      AD500 PMU PROGRAMMING				}
{ Purpose   : This program measures 16 channel of voltages	}
{	      and display them. If the voltage of any channel	}
{	      is greater than a certain level then it close a	}
{	      relay to drive an alarm.				}
{ Remark    : The AD500 has a 16 channel multiplexer in slot	}
{	      0 and 16 channel relay actuator in slot 1. The	}
{	      voltage measurement is done by an HP3478A DMM.	}
{	      The AD500 has an address of 9 and HP3478A has an	}
{	      address of 23.					}
{ Language  : Micro Pascal					}

Program example4(input,output);

{$INCLUDE : 'MSPGPIB.INC'}

procedure iochk;  {  I/O error check routine  }
var
  errno [ origin 16#C000:16#201A] : integer;
begin
  case errno of
    0 : ;
    1 : writeln('TIME OUT !!');
    otherwise
	writeln('INTERFACE ERROR !!');
  end;
end;

var
  oeolstr	     : lstring(8);
  instr,outstr,chstr : lstring(255);
  v		     : array[0..15] of real;
  v_lmt 	     : real;
  i,k,hp3478,ad500   : integer;
  alarm,dum,meas     : integer;
  dumflag	     : boolean;

begin				 { main 		  }
  hp3478:=23;
  ad500:=9;
  v_lmt:=3;			 { Voltage limit	  }
  oeolstr:=chr(13);

  ieseg(16#C000);
  for i:=1 to 24 do writeln;	 { Clear screen 	  }

  ieabort;
  for i:=1 to 10000 do dum:=dum+1-1; { Delay for AD500 reset  }

  ietimout(30000);		 { Setting timeout 5 sec. }
  ieeol(ad500,2,oeolstr,0,10);	 { Init AD500A terminator }

  { Set the DVM and Multiplexer }
  outstr:='F1T3RAN5';
  ieoutput(hp3478,outstr);	 { Send DVM setting string }
  iochk;			 { Error check		   }
  outstr:='DW0,16;DW1,0';
  ieoutput(ad500,outstr);	 { Open Mux & Actuator	   }
  iochk;			 { Error check		   }

  { Measurement start }
  for meas:=1 to 10 do
   begin
     alarm:=0;
     for i:=0 to 15 do
     begin
       dumflag:=encode(chstr,i);
       outstr:='DW0,';
       concat(outstr,chstr);
       ieoutput(ad500,outstr);	  { Close ch.I }
       iochk;
       for k:=1 to 100 do dum:=dum;   { Relay operation delay	 }

       ietrig(hp3478);		  { Trigger the DVM.	     }
       ieenter(hp3478,instr);	  { Enter DVM reading	     }
       iochk;			  { Error check 	     }

       if instr[1]='+' then delete(instr,1,1); { val doesn't like '+' }
       dumflag:=decode(instr,v[i]);
       writeln('CHANNEL ',i:3,v[i]:10:6,'  VOLTS');
       if v[i] > v_lmt	then
       begin
	 writeln('    ALARM !!');
	 alarm:=1;
       end;
     end;
     if alarm=1 then
     begin
       outstr:='DW1,1';
       ieoutput(ad500,outstr);	   { Set the alarm  }
       writeln('Set Alarm ! ');
     end
     else
     begin
       outstr:='DW1,0';
       ieoutput(ad500,outstr);	   { Reset the alarm }
       writeln('Reset Alarm ! ');
     end;
   end;
end.				   { End of program }
